//
// *****************************************************************************
// |          G4LECS -- GEANT4 Low-Energy Compton Scattering package.          |
// |                          R. M. Kippen / LANL                              |
// |                                                                           |
// | G4LECSRaylData.hh                                                         |
// |                                                                           |
// | Classes to read, store, and retrieve tabulated data for Rayleigh          |
// | scattering.                                                               |
// |                                                                           |
// | RMK, DEC-2003. Change g4std headers to standard gcc for G4.6.0            |
// | RMK, MAY-2002. First version (based on G3LECS & G4CompositeEMDataSet.hh)  |
// *****************************************************************************
//
#ifndef G4LECSRAYLDATA_HH
#define G4LECSRAYLDATA_HH 1

#include "globals.hh"
#include "G4DataVector.hh"
#include <vector>

// ----------------------------------------------------------------------------|
// Class holds GLECS data for a single element
// ----------------------------------------------------------------------------|
class G4LECSElemData
{ 
public:

  G4LECSElemData(){};

  ~G4LECSElemData(){};
 
public:

  G4DataVector rayQ  ;   // Grid of Q = 2*E*sin(thet/2) (MeV)
  G4DataVector rayCDF;   // Grid of CDF = Int{FF^2 dq^2}
  size_t nPoints;        // Number of form factor grid points

};

// ----------------------------------------------------------------------------|
// Class holds GLECS data for all elements
// ----------------------------------------------------------------------------|
class G4LECSRaylData
{ 
public:

  G4LECSRaylData(G4int minZ = 1, G4int maxZ = 99);

  G4LECSRaylData(const G4String& dataFile,
		       G4int minZ = 1, G4int maxZ = 99);

  ~G4LECSRaylData();
 
  G4double RandomQval(G4int Z, G4double Qmax) const;

  G4double MaxTabulatedQval(G4int Z) const;

  G4double LogLogIntrp(G4double xval, 
		       const G4DataVector& xdat, 
		       const G4DataVector& ydat) const;

  G4double TableInteg(G4double xlim1, G4double xlim2, 
		       const G4DataVector& xdat, 
		       const G4DataVector& ydat) const;

  void PrintData() const;

private:

  void LoadData(const G4String& fileName);

  std::vector<G4LECSElemData*> components;

  size_t nComponents;

  G4int zMin;
  G4int zMax;

};
 
#endif
