//
// *****************************************************************************
// |          G4LECS -- GEANT4 Low-Energy Compton Scattering package.          |
// |                          R. M. Kippen / LANL                              |
// |                                                                           |
// | G4LECSRayleigh.hh                                                         |
// |                                                                           |
// | Classes to initialize and perform the Rayleigh (coherent) scattering      |
// | process.  Algorithms based on W.R. Nelson et al. The EGS4 Code System.    |
// | SLAC-Report-265, December 1985.                                           |
// |                                                                           |
// | *** Present code for unpolarized radiation only ***                       |
// |                                                                           |
// | RMK, MAY-2002. First version (based on G3LECS & G4LowEnergyRayleigh.hh)   |
// *****************************************************************************
//
//
#ifndef G4LECSRAYLEIGH_HH
#define G4LECSRAYLEIGH_HH 1

#include "globals.hh"
#include "G4VDiscreteProcess.hh"
class G4Track;
class G4Step;
class G4ParticleDefinition;
class G4VParticleChange;
class G4VEMDataSet;
class G4LECSRaylData;
class G4VCrossSectionHandler;

class G4LECSRayleigh : public G4VDiscreteProcess {

public:
  
  G4LECSRayleigh(const G4String& processName ="G4LECSRayleigh");
  
  ~G4LECSRayleigh();

  G4bool IsApplicable(const G4ParticleDefinition&);
  
  void BuildPhysicsTable(const G4ParticleDefinition& photon);
  
  void PrintInfoDefinition();

  G4VParticleChange* PostStepDoIt(const G4Track& aTrack, const G4Step& aStep);
 
  // For testing purpose only
  G4double DumpMeanFreePath(const G4Track& aTrack, 
			    G4double previousStepSize, 
			    G4ForceCondition* condition) 
  { return GetMeanFreePath(aTrack, previousStepSize, condition); }

protected:

  G4double GetMeanFreePath(const G4Track& aTrack, 
			   G4double previousStepSize, 
			   G4ForceCondition* condition);

private:

  // Hide copy constructor and assignment operator as private 
  G4LECSRayleigh& operator=(const G4LECSRayleigh &right);
  G4LECSRayleigh(const G4LECSRayleigh& );
  
  G4double lowEnergyLimit;  // low energy limit  applied to the process
  G4double highEnergyLimit; // high energy limit applied to the process

  G4VEMDataSet* meanFreePathTable;

  G4LECSRaylData* formFactorData;

  G4VCrossSectionHandler* crossSectionHandler;
 
  const G4double intrinsicLowEnergyLimit; // intrinsic validity range
  const G4double intrinsicHighEnergyLimit;

};

#endif
